@@ -7,7 +7,7 @@ Fasta file -> Faidx -> Fasta -> FastaRecord -> Sequence
 from __future__ import division
 import os
 from os.path import getmtime
-from six import PY2, PY3, string_types
+from six import PY2, PY3, string_types, integer_types
 from six.moves import zip_longest
 try:
     from collections import OrderedDict
@@ -18,10 +18,11 @@ import re
 import string
 import warnings
 from math import ceil
+from threading import Lock
 
 dna_bases = re.compile(r'([ACTGNactgnYRWSKMDVHBXyrwskmdvhbx]+)')
 
-__version__ = '0.4.7.1'
+__version__ = '0.4.8.1'
 
 
 class FastaIndexingError(Exception):
@@ -137,7 +138,7 @@ class Sequence(object):
                 start = self_start + slice_start
                 end = self_start + slice_stop + correction_factor
             return self.__class__(self.name, self.seq[n], start, end, self.comp)
-        elif isinstance(n, int):
+        elif isinstance(n, integer_types):
             if n < 0:
                 n = len(self) + n
             if self.start:
@@ -268,7 +269,7 @@ class Faidx(object):
                  as_raw=False, strict_bounds=False, read_ahead=None,
                  mutable=False, split_char=None, filt_function=None,
                  one_based_attributes=True,
-                 sequence_always_upper=False):
+                 sequence_always_upper=False, rebuild=True):
         """
         filename: name of fasta file
         key_function: optional callback function which should return a unique
@@ -291,23 +292,32 @@ class Faidx(object):
         self.one_based_attributes = one_based_attributes
         self.sequence_always_upper = sequence_always_upper
         self.index = OrderedDict()
+        self.lock = Lock()
         self.buffer = dict((('seq', None), ('name', None), ('start', None), ('end', None)))
-        if not read_ahead or isinstance(read_ahead, int):
+        if not read_ahead or isinstance(read_ahead, integer_types):
             self.read_ahead = read_ahead
-        elif not isinstance(read_ahead, int):
+        elif not isinstance(read_ahead, integer_types):
             raise ValueError("read_ahead value must be int, not {0}".format(type(read_ahead)))
 
         self.mutable = mutable
-
-        if os.path.exists(self.indexname) and getmtime(self.indexname) >= getmtime(self.filename):
-            self.read_fai(split_char)
-        else:
+        with self.lock:  # lock around index generation so only one thread calls method
             try:
-                self.build_index()
+                if os.path.exists(self.indexname) and getmtime(self.indexname) >= getmtime(self.filename):
+                    self.read_fai(split_char)
+                elif os.path.exists(self.indexname) and getmtime(self.indexname) < getmtime(self.filename) and not rebuild:
+                    self.read_fai(split_char)
+                    warnings.warn("Index file {0} is older than FASTA file {1}.".format(self.indexname, self.filename), RuntimeWarning)
+                else:
+                    self.build_index()
+                    self.read_fai(split_char)
             except FastaIndexingError as e:
                 os.remove(self.indexname)
+                self.file.close()
                 raise FastaIndexingError(e)
-            self.read_fai(split_char)
+            except Exception:
+                # Handle potential exceptions other than 'FastaIndexingError'
+                self.file.close()
+                raise
 
     def __contains__(self, region):
         if not self.buffer['name']:
@@ -349,66 +359,70 @@ class Faidx(object):
                 self.index.pop(dup, None)
 
     def build_index(self):
-        with open(self.filename, 'r') as fastafile:
-            with open(self.indexname, 'w') as indexfile:
-                rname = None  # reference sequence name
-                offset = 0  # binary offset of end of current line
-                rlen = 0  # reference character length
-                blen = None  # binary line length (includes newline)
-                clen = None  # character line length
-                bad_lines = []  # lines > || < than blen
-                thisoffset = offset
-                for i, line in enumerate(fastafile):
-                    line_blen = len(line)
-                    line_clen = len(line.rstrip('\n\r'))
-                    # write an index line
-                    if line[0] == '>':
-                        valid_entry = check_bad_lines(rname, bad_lines, i - 1)
-                        if valid_entry and i > 0:
-                            indexfile.write("{0}\t{1:d}\t{2:d}\t{3:d}\t{4:d}\n".format(rname, rlen, thisoffset, clen, blen))
-                        elif not valid_entry:
-                            raise FastaIndexingError("Line length of fasta"
-                                                     " file is not "
-                                                     "consistent! "
-                                                     "Inconsistent line found in >{0} at "
-                                                     "line {1:n}.".format(rname, bad_lines[0][0] + 1))
-                        blen = None
-                        rlen = 0
-                        clen = None
-                        bad_lines = []
-                        try:  # must catch empty deflines
-                            rname = line.rstrip('\n\r')[1:].split()[0]  # remove comments
-                        except IndexError:
-                            raise FastaIndexingError("Bad sequence name %s at line %s." % (line.rstrip('\n\r'), str(i)))
-                        offset += line_blen
-                        thisoffset = offset
-                    else:  # check line and advance offset
-                        if not blen:
-                            blen = line_blen
-                        if not clen:
-                            clen = line_clen
-                        # only one short line should be allowed
-                        # before we hit the next header, and it
-                        # should be the last line in the entry
-                        if line_blen != blen or line_blen == 1:
-                            bad_lines.append((i, line_blen))
-                        offset += line_blen
-                        rlen += line_clen
+        try:
+            with open(self.filename, 'r') as fastafile:
+                with open(self.indexname, 'w') as indexfile:
+                    rname = None  # reference sequence name
+                    offset = 0  # binary offset of end of current line
+                    rlen = 0  # reference character length
+                    blen = None  # binary line length (includes newline)
+                    clen = None  # character line length
+                    bad_lines = []  # lines > || < than blen
+                    thisoffset = offset
+                    for i, line in enumerate(fastafile):
+                        line_blen = len(line)
+                        line_clen = len(line.rstrip('\n\r'))
+                        # write an index line
+                        if line[0] == '>':
+                            valid_entry = check_bad_lines(rname, bad_lines, i - 1)
+                            if valid_entry and i > 0:
+                                indexfile.write("{0}\t{1:d}\t{2:d}\t{3:d}\t{4:d}\n".format(rname, rlen, thisoffset, clen, blen))
+                            elif not valid_entry:
+                                raise FastaIndexingError("Line length of fasta"
+                                                         " file is not "
+                                                         "consistent! "
+                                                         "Inconsistent line found in >{0} at "
+                                                         "line {1:n}.".format(rname, bad_lines[0][0] + 1))
+                            blen = None
+                            rlen = 0
+                            clen = None
+                            bad_lines = []
+                            try:  # must catch empty deflines
+                                rname = line.rstrip('\n\r')[1:].split()[0]  # remove comments
+                            except IndexError:
+                                raise FastaIndexingError("Bad sequence name %s at line %s." % (line.rstrip('\n\r'), str(i)))
+                            offset += line_blen
+                            thisoffset = offset
+                        else:  # check line and advance offset
+                            if not blen:
+                                blen = line_blen
+                            if not clen:
+                                clen = line_clen
+                            # only one short line should be allowed
+                            # before we hit the next header, and it
+                            # should be the last line in the entry
+                            if line_blen != blen or line_blen == 1:
+                                bad_lines.append((i, line_blen))
+                            offset += line_blen
+                            rlen += line_clen
 
-                # write the final index line
-                valid_entry = check_bad_lines(rname, bad_lines, i)  # advance index since we're at the end of the file
-                if not valid_entry:
-                    raise FastaIndexingError("Line length of fasta"
-                                             " file is not "
-                                             "consistent! "
-                                             "Inconsistent line found in >{0} at "
-                                             "line {1:n}.".format(rname, bad_lines[0][0] + 1))
-                indexfile.write("{0:s}\t{1:d}\t{2:d}\t{3:d}\t{4:d}\n".format(rname, rlen, thisoffset, clen, blen))
+                    # write the final index line
+                    valid_entry = check_bad_lines(rname, bad_lines, i)  # advance index since we're at the end of the file
+                    if not valid_entry:
+                        raise FastaIndexingError("Line length of fasta"
+                                                 " file is not "
+                                                 "consistent! "
+                                                 "Inconsistent line found in >{0} at "
+                                                 "line {1:n}.".format(rname, bad_lines[0][0] + 1))
+                    indexfile.write("{0:s}\t{1:d}\t{2:d}\t{3:d}\t{4:d}\n".format(rname, rlen, thisoffset, clen, blen))
+        except IOError:
+            raise IOError("%s may not be writable. Please use Fasta(rebuild=False), Faidx(rebuild=False) or faidx --no-rebuild." % self.indexname)
 
     def write_fai(self):
-        with open(self.indexname, 'w') as outfile:
-            for k, v in self.index.items():
-                outfile.write('\t'.join([k, str(v)]))
+        with self.lock:
+            with open(self.indexname, 'w') as outfile:
+                for k, v in self.index.items():
+                    outfile.write('\t'.join([k, str(v)]))
 
     def from_buffer(self, start, end):
         i_start = start - self.buffer['start']  # want [0, 1) coordinates from [1, 1] coordinates
@@ -444,8 +458,8 @@ class Faidx(object):
         4. Seek to start position, taking newlines into account
         5. Read to end position, return sequence
         """
-        assert isinstance(start, int)
-        assert isinstance(end, int)
+        assert isinstance(start, integer_types)
+        assert isinstance(end, integer_types)
         try:
             i = self.index[rname]
         except KeyError:
@@ -463,20 +477,23 @@ class Faidx(object):
         newlines_inside = newlines_to_end - newlines_before
         seq_blen = newlines_inside + seq_len
         bstart = i.offset + newlines_before + start0
-        self.file.seek(bstart)
 
-        if bstart + seq_blen > i.bend and not self.strict_bounds:
-            seq_blen = i.bend - bstart
-        elif bstart + seq_blen > i.bend and self.strict_bounds:
-            raise FetchError("Requested end coordinate {0:n} outside of {1}. "
-                             "\n".format(end, rname))
-        if seq_blen > 0:
-            seq = self.file.read(seq_blen).decode()
-        elif seq_blen <= 0 and not self.strict_bounds:
-            seq = ''
-        elif seq_blen <= 0 and self.strict_bounds:
-            raise FetchError("Requested coordinates start={0:n} end={1:n} are "
-                             "invalid.\n".format(start, end))
+        with self.lock:
+            self.file.seek(bstart)
+
+            if bstart + seq_blen > i.bend and not self.strict_bounds:
+                seq_blen = i.bend - bstart
+            elif bstart + seq_blen > i.bend and self.strict_bounds:
+                raise FetchError("Requested end coordinate {0:n} outside of {1}. "
+                                 "\n".format(end, rname))
+            if seq_blen > 0:
+                seq = self.file.read(seq_blen).decode()
+            elif seq_blen <= 0 and not self.strict_bounds:
+                seq = ''
+            elif seq_blen <= 0 and self.strict_bounds:
+                raise FetchError("Requested coordinates start={0:n} end={1:n} are "
+                                 "invalid.\n".format(start, end))
+
         if not internals:
             return seq.replace('\n', '')
         else:
@@ -508,21 +525,23 @@ class Faidx(object):
         if not self.mutable:
             raise IOError("Write attempted for immutable Faidx instance. Set mutable=True to modify original FASTA.")
         file_seq, internals = self.from_file(rname, start, end, internals=True)
-        if len(seq) != len(file_seq) - internals['newlines_inside']:
-            raise IOError("Specified replacement sequence needs to have the same length as original.")
-        elif len(seq) == len(file_seq) - internals['newlines_inside']:
-            line_len = internals['i'].lenc
-            self.file.seek(internals['bstart'])
-            if internals['newlines_inside'] == 0:
-                self.file.write(seq.encode())
-            elif internals['newlines_inside'] > 0:
-                n = 0
-                m = file_seq.index('\n')
-                while m < len(seq):
-                    self.file.write(''.join([seq[n:m], '\n']).encode())
-                    n = m
-                    m += line_len
-                self.file.write(seq[n:].encode())
+
+        with self.lock:
+            if len(seq) != len(file_seq) - internals['newlines_inside']:
+                raise IOError("Specified replacement sequence needs to have the same length as original.")
+            elif len(seq) == len(file_seq) - internals['newlines_inside']:
+                line_len = internals['i'].lenc
+                self.file.seek(internals['bstart'])
+                if internals['newlines_inside'] == 0:
+                    self.file.write(seq.encode())
+                elif internals['newlines_inside'] > 0:
+                    n = 0
+                    m = file_seq.index('\n')
+                    while m < len(seq):
+                        self.file.write(''.join([seq[n:m], '\n']).encode())
+                        n = m
+                        m += line_len
+                    self.file.write(seq[n:].encode())
 
     def close(self):
         self.__exit__()
@@ -556,7 +575,7 @@ class FastaRecord(object):
                     start = len(self) + start
                 return self._fa.get_seq(self.name, start + 1, stop)[::step]
 
-            elif isinstance(n, int):
+            elif isinstance(n, integer_types):
                 if n < 0:
                     n = len(self) + n
                 return self._fa.get_seq(self.name, n + 1, n + 1)
@@ -632,7 +651,7 @@ class MutableFastaRecord(FastaRecord):
                     start = len(self) + start
                 self._fa.faidx.to_file(self.name, start + 1, stop, value)
 
-            elif isinstance(n, int):
+            elif isinstance(n, integer_types):
                 if n < 0:
                     n = len(self) + n
                 return self._fa.faidx.to_file(self.name, n + 1, n + 1, value)
@@ -644,7 +663,7 @@ class Fasta(object):
     def __init__(self, filename, default_seq=None, key_function=None, as_raw=False,
                  strict_bounds=False, read_ahead=None, mutable=False, split_char=None,
                  filt_function=None, one_based_attributes=True,
-                 sequence_always_upper=False):
+                 sequence_always_upper=False, rebuild=True):
         """
         An object that provides a pygr compatible interface.
         filename: name of fasta file
@@ -655,7 +674,7 @@ class Fasta(object):
                            default_seq=default_seq, strict_bounds=strict_bounds,
                            read_ahead=read_ahead, mutable=mutable, split_char=split_char,
                            filt_function=filt_function, one_based_attributes=one_based_attributes,
-                           sequence_always_upper=sequence_always_upper)
+                           sequence_always_upper=sequence_always_upper, rebuild=rebuild)
         self.keys = self.faidx.index.keys
         if not self.mutable:
             self.records = dict([(rname, FastaRecord(rname, self)) for rname in self.keys()])
@@ -668,7 +687,7 @@ class Fasta(object):
 
     def __getitem__(self, rname):
         """Return a chromosome by its name, or its numerical index."""
-        if isinstance(rname, int):
+        if isinstance(rname, integer_types):
             rname = tuple(self.keys())[rname]
         try:
             return self.records[rname]
@@ -759,7 +778,7 @@ class FastaVariant(Fasta):
         else:
             seq_mut = list(seq.seq)
             del seq.seq
-        var = self.vcf.fetch(name, start, end)
+        var = self.vcf.fetch(name, start - 1, end)
         for record in var:
             if record.is_snp:  # skip indels
                 sample = record.genotype(self.sample)
